
# parsetab.py
# This file is automatically generated. Do not edit.
_tabversion = '3.8'

_lr_method = 'LALR'

_lr_signature = 'D349F105D4176011A9739F6739BFAE91'
    
_lr_action_items = {'STAR':([33,84,152,153,154,155,],[40,92,161,-96,163,-95,]),'MA':([118,119,120,137,],[-99,-98,136,136,]),'TIME_PARAMETER':([169,172,190,],[178,178,178,]),'EQUAL':([96,97,98,99,102,176,178,179,180,182,],[108,109,110,111,116,188,191,192,193,194,]),'SQRPARAN':([63,67,68,69,70,88,136,138,140,141,142,143,150,151,167,173,175,183,185,206,207,],[77,80,-56,-57,-52,-53,-31,-12,-14,-11,-15,-13,159,160,174,186,187,196,199,208,209,]),'MEDIUM_SPECIFICATION':([30,36,37,47,117,],[35,-44,35,-45,-49,]),'RECEIVING_CURRENT_PARAMETER':([169,172,190,],[179,179,179,]),'ARROWLEFT':([146,147,149,174,186,187,199,],[155,-76,-77,-82,-83,-84,-85,]),'COMM_PLUS':([183,],[197,]),'SET':([12,14,15,19,49,50,],[13,-29,13,-30,-33,-32,]),'SEMICOLON':([38,39,100,115,118,119,121,122,123,124,125,126,127,129,130,131,134,135,136,138,139,140,141,142,143,159,160,162,163,164,165,166,174,177,181,184,186,187,196,199,200,201,202,203,204,205,208,209,],[49,50,112,133,-99,-98,-9,-20,-28,-19,-17,-6,-4,-1,-8,-3,-58,-22,-31,-12,-5,-14,-11,-15,-13,-27,-7,-81,-79,-78,-80,171,-82,189,-89,198,-83,-84,-86,-85,-91,-90,-2,-16,-18,-10,-88,-87,]),'RPARAN':([31,32,33,40,42,43,64,84,92,94,],[38,39,41,51,52,-100,-101,93,105,106,]),'TOPOLOGY_SPECIFICATION':([101,112,133,],[114,-54,-55,]),'SENDING_CURRENT_DEFAULT_PARAMETER':([87,101,112,133,],[98,98,-54,-55,]),'COMMA':([42,43,64,67,68,69,70,88,94,118,119,122,123,124,127,130,131,135,136,138,139,140,141,142,143,159,160,177,181,184,200,201,202,203,204,205,],[53,-100,-101,79,-56,-57,-52,-53,53,-99,-98,-20,-28,-19,-4,-8,-3,-22,-31,-12,-5,-14,-11,-15,-13,-27,-7,190,-89,190,-91,-90,-2,-16,-18,-10,]),'KBYTEPS':([118,119,128,144,],[-99,-98,140,140,]),'SENDING_CURRENT_PARAMETER':([169,172,190,],[182,182,182,]),'INTEGER':([45,91,108,109,110,111,116,156,168,170,173,188,191,192,193,194,195,197,],[61,61,118,118,118,118,118,167,175,167,185,118,118,118,118,118,206,207,]),'IDENTIFIER':([6,16,22,27,33,46,53,55,57,79,84,85,108,109,110,111,132,145,148,152,153,154,155,157,171,189,191,192,193,194,198,],[9,21,28,32,43,63,64,69,72,69,43,95,123,123,123,130,147,-62,147,-94,-96,164,-95,-63,-66,-72,130,123,123,123,-67,]),'$end':([2,3,5,11,],[-92,0,-93,-21,]),'RECEIVING_CURRENT_DEFAULT_PARAMETER':([87,101,112,133,],[96,96,-54,-55,]),'COLON':([156,161,162,163,164,165,166,167,170,174,186,187,196,199,208,209,],[168,169,-81,-79,-78,-80,172,173,168,-82,-83,-84,-86,-85,-88,-87,]),'VERSIONS_SPECIFICATION':([0,2,3,5,11,],[1,-92,1,-93,-21,]),'ARROWBOTH':([146,147,149,174,186,187,199,],[153,-76,-77,-82,-83,-84,-85,]),'Q_PARAMETER':([169,172,190,],[176,176,176,]),'HOST':([13,17,],[16,22,]),'VERSION':([4,7,8,10,26,29,],[6,6,-23,-24,-25,-26,]),'BLOCKOPEN':([1,9,25,34,41,44,51,52,56,76,77,80,83,93,105,106,114,],[4,12,30,45,-46,54,-47,-48,71,-43,87,-50,91,-73,-74,-75,132,]),'COMMUNICATION_SPECIFICATION':([18,20,24,62,66,75,78,82,89,],[25,-34,-35,-36,-37,-39,-40,-38,-41,]),'SQLPARAN':([35,44,76,90,123,130,147,164,],[46,55,-43,55,137,144,156,170,]),'BLOCKCLOSE':([7,8,10,18,20,23,24,26,29,36,37,45,47,48,54,58,59,60,61,62,65,66,71,74,75,76,78,80,81,82,83,86,89,90,93,103,104,105,106,107,113,117,145,148,157,158,171,189,198,],[11,-23,-24,26,-34,29,-35,-25,-26,-44,48,62,-45,-42,66,-64,75,-59,76,-36,78,-37,82,-60,-39,-43,-40,-50,89,-38,-68,-65,-41,-69,-73,117,-70,-74,-75,-71,-51,-49,-62,158,-63,-61,-66,-72,-67,]),'ARROWRIGHT':([58,76,80,83,90,93,104,105,106,146,147,149,174,186,187,199,],[73,-43,-50,-68,-69,-73,-70,-74,-75,152,-76,-77,-82,-83,-84,-85,]),'LPARAN':([21,28,72,95,],[27,33,84,84,]),'MSPBIT':([118,119,128,144,],[-99,-98,138,138,]),'RUN':([14,15,18,19,20,24,45,49,50,54,58,59,60,62,65,66,71,73,74,75,76,78,80,81,82,83,86,89,90,93,104,105,106,107,],[-29,17,17,-30,-34,-35,57,-33,-32,57,-64,57,-59,-36,57,-37,57,85,-60,-39,-43,-40,-50,57,-38,-68,-65,-41,-69,-73,-70,-74,-75,-71,]),'MBYTEPS':([118,119,128,144,],[-99,-98,142,142,]),'COMM_MINUS':([183,],[195,]),'FLOAT':([108,109,110,111,116,188,191,192,193,194,],[119,119,119,119,119,119,119,119,119,119,]),'QUALIFIED_IDENTIFIER':([27,55,79,],[31,68,68,]),'LISTENING_CURRENT_PARAMETER':([169,172,190,],[180,180,180,]),'I_INDEX':([170,],[183,]),'LISTENING_CURRENT_DEFAULT_PARAMETER':([87,101,112,133,],[97,97,-54,-55,]),'TIME_DEFAULT_PARAMETER':([87,101,112,133,],[99,99,-54,-55,]),'QUALITY_DEFAULT_PARAMETER':([87,101,112,133,],[102,102,-54,-55,]),'MS':([118,119,128,144,],[-99,-98,141,141,]),'MSPBYTE':([118,119,128,144,],[-99,-98,143,143,]),}

_lr_action = {}
for _k, _v in _lr_action_items.items():
   for _x,_y in zip(_v[0],_v[1]):
      if not _x in _lr_action:  _lr_action[_x] = {}
      _lr_action[_x][_k] = _y
del _lr_action_items

_lr_goto_items = {'version_topology_rule_left_hosts':([132,148,],[146,146,]),'version_run_hosts':([15,],[18,]),'version_topology_rule_parameters':([169,172,],[177,184,]),'version_run_processes':([45,54,71,],[59,65,81,]),'number':([108,109,110,111,116,188,191,192,193,194,],[120,120,120,128,134,200,128,120,120,120,]),'version_topology_host_with_i_index':([154,],[162,]),'version_repetition':([34,83,],[44,90,]),'version_run_process':([45,54,59,65,71,81,],[60,60,74,74,60,74,]),'version_topology_rule_parameter':([169,172,190,],[181,181,201,]),'version_topology_arrow':([146,],[154,]),'version_medium_default_parameters':([87,],[101,]),'version_comm_time_metric':([111,191,],[131,131,]),'version_medium_elements':([87,],[103,]),'version_comm_specifications':([30,],[37,]),'version_communication':([18,],[23,]),'version_repetition_channels_list':([55,],[67,]),'version_run_host':([15,18,],[20,24,]),'specification':([0,3,],[2,5,]),'versions_list':([4,],[7,]),'version_comm_current_metric_unit':([120,137,],[135,150,]),'version':([4,7,],[8,10,]),'version_comm_time_value':([111,191,],[129,202,]),'version_run_process_follower':([73,],[86,]),'version_medium_default_parameter':([87,101,],[100,115,]),'metrics_sets':([12,],[15,]),'version_repetition_channels':([44,90,],[56,104,]),'version_topology_rule_right_hosts':([154,],[166,]),'version_topology_rule':([132,148,],[145,157,]),'version_run_process_base':([45,54,59,65,71,81,],[58,58,58,58,58,58,]),'metrics_set':([12,15,],[14,19,]),'version_comm_time_metric_unit':([128,144,],[139,151,]),'identifiers_list':([33,84,],[42,94,]),'version_comm_current_algorithm':([108,109,110,192,193,194,],[122,122,122,122,122,122,]),'configuration':([0,],[3,]),'version_repetition_channel':([55,79,],[70,88,]),'version_comm_current_metric':([108,109,110,192,193,194,],[124,124,124,124,124,124,]),'version_subprocesses_list':([72,95,],[83,107,]),'version_medium_topology':([101,],[113,]),'version_topology_host_with_indicies':([132,148,154,],[149,149,165,]),'version_comm_specification':([30,37,],[36,47,]),'version_channels':([28,],[34,]),'version_topology_rules_list':([132,],[148,]),'version_comm_current_value':([108,109,110,192,193,194,],[121,125,126,203,204,205,]),'version_comm_time_algorithm':([111,191,],[127,127,]),}

_lr_goto = {}
for _k, _v in _lr_goto_items.items():
   for _x, _y in zip(_v[0], _v[1]):
       if not _x in _lr_goto: _lr_goto[_x] = {}
       _lr_goto[_x][_k] = _y
del _lr_goto_items
_lr_productions = [
  ("S' -> configuration","S'",1,None,None,None),
  ('version_medium_default_parameter -> TIME_DEFAULT_PARAMETER EQUAL version_comm_time_value','version_medium_default_parameter',3,'p_version_time_default_parameter','parser.py',116),
  ('version_topology_rule_parameter -> TIME_PARAMETER EQUAL version_comm_time_value','version_topology_rule_parameter',3,'p_version_topology_rule_time_parameter','parser.py',122),
  ('version_comm_time_value -> version_comm_time_metric','version_comm_time_value',1,'p_version_comm_time_value','parser.py',128),
  ('version_comm_time_value -> version_comm_time_algorithm','version_comm_time_value',1,'p_version_comm_time_value','parser.py',129),
  ('version_comm_time_metric -> number version_comm_time_metric_unit','version_comm_time_metric',2,'p_version_comm_time_metric','parser.py',135),
  ('version_medium_default_parameter -> SENDING_CURRENT_DEFAULT_PARAMETER EQUAL version_comm_current_value','version_medium_default_parameter',3,'p_version_sending_current_default_parameter','parser.py',145),
  ('version_comm_time_algorithm -> IDENTIFIER SQLPARAN version_comm_time_metric_unit SQRPARAN','version_comm_time_algorithm',4,'p_version_comm_time_algorithm','parser.py',145),
  ('version_comm_time_algorithm -> IDENTIFIER','version_comm_time_algorithm',1,'p_version_comm_time_algorithm','parser.py',146),
  ('version_medium_default_parameter -> RECEIVING_CURRENT_DEFAULT_PARAMETER EQUAL version_comm_current_value','version_medium_default_parameter',3,'p_version_receiving_current_default_parameter','parser.py',151),
  ('version_topology_rule_parameter -> SENDING_CURRENT_PARAMETER EQUAL version_comm_current_value','version_topology_rule_parameter',3,'p_version_topology_rule_sending_current_parameter','parser.py',157),
  ('version_comm_time_metric_unit -> MS','version_comm_time_metric_unit',1,'p_version_comm_time_metric_unit','parser.py',159),
  ('version_comm_time_metric_unit -> MSPBIT','version_comm_time_metric_unit',1,'p_version_comm_time_metric_unit','parser.py',160),
  ('version_comm_time_metric_unit -> MSPBYTE','version_comm_time_metric_unit',1,'p_version_comm_time_metric_unit','parser.py',161),
  ('version_comm_time_metric_unit -> KBYTEPS','version_comm_time_metric_unit',1,'p_version_comm_time_metric_unit','parser.py',162),
  ('version_comm_time_metric_unit -> MBYTEPS','version_comm_time_metric_unit',1,'p_version_comm_time_metric_unit','parser.py',163),
  ('version_topology_rule_parameter -> RECEIVING_CURRENT_PARAMETER EQUAL version_comm_current_value','version_topology_rule_parameter',3,'p_version_topology_rule_receiving_current_parameter','parser.py',163),
  ('version_medium_default_parameter -> LISTENING_CURRENT_DEFAULT_PARAMETER EQUAL version_comm_current_value','version_medium_default_parameter',3,'p_version_listening_current_default_parameter','parser.py',169),
  ('version_topology_rule_parameter -> LISTENING_CURRENT_PARAMETER EQUAL version_comm_current_value','version_topology_rule_parameter',3,'p_version_topology_rule_listening_current_parameter','parser.py',175),
  ('version_comm_current_value -> version_comm_current_metric','version_comm_current_value',1,'p_version_comm_current_value','parser.py',181),
  ('version_comm_current_value -> version_comm_current_algorithm','version_comm_current_value',1,'p_version_comm_current_value','parser.py',182),
  ('specification -> VERSIONS_SPECIFICATION BLOCKOPEN versions_list BLOCKCLOSE','specification',4,'p_versions_specification','versions.py',184),
  ('version_comm_current_metric -> number version_comm_current_metric_unit','version_comm_current_metric',2,'p_version_comm_current_metric','parser.py',188),
  ('versions_list -> version','versions_list',1,'p_versions_list','versions.py',191),
  ('versions_list -> versions_list version','versions_list',2,'p_versions_list','versions.py',192),
  ('version -> VERSION IDENTIFIER BLOCKOPEN metrics_sets version_run_hosts BLOCKCLOSE','version',6,'p_version','versions.py',198),
  ('version -> VERSION IDENTIFIER BLOCKOPEN metrics_sets version_run_hosts version_communication BLOCKCLOSE','version',7,'p_version','versions.py',199),
  ('version_comm_current_algorithm -> IDENTIFIER SQLPARAN version_comm_current_metric_unit SQRPARAN','version_comm_current_algorithm',4,'p_version_comm_current_algorithm','parser.py',198),
  ('version_comm_current_algorithm -> IDENTIFIER','version_comm_current_algorithm',1,'p_version_comm_current_algorithm','parser.py',199),
  ('metrics_sets -> metrics_set','metrics_sets',1,'p_metrics_sets','versions.py',205),
  ('metrics_sets -> metrics_sets metrics_set','metrics_sets',2,'p_metrics_sets','versions.py',206),
  ('version_comm_current_metric_unit -> MA','version_comm_current_metric_unit',1,'p_version_comm_current_metric_unit','parser.py',212),
  ('metrics_set -> SET HOST IDENTIFIER LPARAN IDENTIFIER RPARAN SEMICOLON','metrics_set',7,'p_metrics_set','versions.py',217),
  ('metrics_set -> SET HOST IDENTIFIER LPARAN QUALIFIED_IDENTIFIER RPARAN SEMICOLON','metrics_set',7,'p_metrics_set','versions.py',218),
  ('version_run_hosts -> version_run_host','version_run_hosts',1,'p_version_run_hosts','versions.py',224),
  ('version_run_hosts -> version_run_hosts version_run_host','version_run_hosts',2,'p_version_run_hosts','versions.py',225),
  ('version_run_host -> RUN HOST IDENTIFIER version_channels BLOCKOPEN BLOCKCLOSE','version_run_host',6,'p_version_run_host','versions.py',236),
  ('version_run_host -> RUN HOST IDENTIFIER version_channels version_repetition BLOCKOPEN BLOCKCLOSE','version_run_host',7,'p_version_run_host','versions.py',237),
  ('version_run_host -> RUN HOST IDENTIFIER version_channels version_repetition version_repetition_channels BLOCKOPEN BLOCKCLOSE','version_run_host',8,'p_version_run_host','versions.py',238),
  ('version_run_host -> RUN HOST IDENTIFIER version_channels BLOCKOPEN version_run_processes BLOCKCLOSE','version_run_host',7,'p_version_run_host','versions.py',239),
  ('version_run_host -> RUN HOST IDENTIFIER version_channels version_repetition BLOCKOPEN version_run_processes BLOCKCLOSE','version_run_host',8,'p_version_run_host','versions.py',240),
  ('version_run_host -> RUN HOST IDENTIFIER version_channels version_repetition version_repetition_channels BLOCKOPEN version_run_processes BLOCKCLOSE','version_run_host',9,'p_version_run_host','versions.py',241),
  ('version_communication -> COMMUNICATION_SPECIFICATION BLOCKOPEN version_comm_specifications BLOCKCLOSE','version_communication',4,'p_version_communication','communication.py',241),
  ('version_repetition -> BLOCKOPEN INTEGER BLOCKCLOSE','version_repetition',3,'p_version_repetition','versions.py',247),
  ('version_comm_specifications -> version_comm_specification','version_comm_specifications',1,'p_version_comm_specifications','communication.py',250),
  ('version_comm_specifications -> version_comm_specifications version_comm_specification','version_comm_specifications',2,'p_version_comm_specifications','communication.py',251),
  ('version_channels -> LPARAN RPARAN','version_channels',2,'p_version_channels','versions.py',253),
  ('version_channels -> LPARAN STAR RPARAN','version_channels',3,'p_version_channels','versions.py',254),
  ('version_channels -> LPARAN identifiers_list RPARAN','version_channels',3,'p_version_channels','versions.py',255),
  ('version_comm_specification -> MEDIUM_SPECIFICATION SQLPARAN IDENTIFIER SQRPARAN BLOCKOPEN version_medium_elements BLOCKCLOSE','version_comm_specification',7,'p_version_medium_specification','communication.py',261),
  ('version_repetition_channels -> SQLPARAN version_repetition_channels_list SQRPARAN','version_repetition_channels',3,'p_version_repetition_channels','versions.py',267),
  ('version_medium_elements -> version_medium_default_parameters version_medium_topology','version_medium_elements',2,'p_version_medium_elements','communication.py',269),
  ('version_repetition_channels_list -> version_repetition_channel','version_repetition_channels_list',1,'p_version_repetition_channels_list','versions.py',273),
  ('version_repetition_channels_list -> version_repetition_channels_list COMMA version_repetition_channel','version_repetition_channels_list',3,'p_version_repetition_channels_list','versions.py',274),
  ('version_medium_default_parameters -> version_medium_default_parameter SEMICOLON','version_medium_default_parameters',2,'p_version_medium_default_parameters','communication.py',278),
  ('version_medium_default_parameters -> version_medium_default_parameters version_medium_default_parameter SEMICOLON','version_medium_default_parameters',3,'p_version_medium_default_parameters','communication.py',279),
  ('version_repetition_channel -> QUALIFIED_IDENTIFIER','version_repetition_channel',1,'p_version_repetition_channel','versions.py',285),
  ('version_repetition_channel -> IDENTIFIER','version_repetition_channel',1,'p_version_repetition_channel','versions.py',286),
  ('version_medium_default_parameter -> QUALITY_DEFAULT_PARAMETER EQUAL number','version_medium_default_parameter',3,'p_version_quality_default_parameter','communication.py',287),
  ('version_run_processes -> version_run_process','version_run_processes',1,'p_version_run_processes','versions.py',292),
  ('version_run_processes -> version_run_processes version_run_process','version_run_processes',2,'p_version_run_processes','versions.py',293),
  ('version_medium_topology -> TOPOLOGY_SPECIFICATION BLOCKOPEN version_topology_rules_list BLOCKCLOSE','version_medium_topology',4,'p_version_medium_topology','communication.py',293),
  ('version_topology_rules_list -> version_topology_rule','version_topology_rules_list',1,'p_version_topology_rules_list','communication.py',299),
  ('version_topology_rules_list -> version_topology_rules_list version_topology_rule','version_topology_rules_list',2,'p_version_topology_rules_list','communication.py',300),
  ('version_run_process -> version_run_process_base','version_run_process',1,'p_version_run_process','versions.py',304),
  ('version_run_process -> version_run_process_base ARROWRIGHT version_run_process_follower','version_run_process',3,'p_version_run_process','versions.py',305),
  ('version_topology_rule -> version_topology_rule_left_hosts version_topology_arrow version_topology_rule_right_hosts SEMICOLON','version_topology_rule',4,'p_version_topology_rule_point_to_point','communication.py',311),
  ('version_topology_rule -> version_topology_rule_left_hosts version_topology_arrow version_topology_rule_right_hosts COLON version_topology_rule_parameters SEMICOLON','version_topology_rule',6,'p_version_topology_rule_point_to_point','communication.py',312),
  ('version_run_process_base -> RUN IDENTIFIER version_subprocesses_list','version_run_process_base',3,'p_version_run_process_base','versions.py',311),
  ('version_run_process_base -> RUN IDENTIFIER version_subprocesses_list version_repetition','version_run_process_base',4,'p_version_run_process_base','versions.py',312),
  ('version_run_process_base -> RUN IDENTIFIER version_subprocesses_list version_repetition version_repetition_channels','version_run_process_base',5,'p_version_run_process_base','versions.py',313),
  ('version_run_process_follower -> RUN IDENTIFIER version_subprocesses_list','version_run_process_follower',3,'p_version_run_process_follower','versions.py',319),
  ('version_topology_rule -> version_topology_rule_left_hosts ARROWRIGHT STAR COLON version_topology_rule_parameters SEMICOLON','version_topology_rule',6,'p_version_topology_rule_boradcast','communication.py',321),
  ('version_subprocesses_list -> LPARAN RPARAN','version_subprocesses_list',2,'p_version_subprocesses_list','versions.py',325),
  ('version_subprocesses_list -> LPARAN STAR RPARAN','version_subprocesses_list',3,'p_version_subprocesses_list','versions.py',326),
  ('version_subprocesses_list -> LPARAN identifiers_list RPARAN','version_subprocesses_list',3,'p_version_subprocesses_list','versions.py',327),
  ('version_topology_rule_left_hosts -> IDENTIFIER','version_topology_rule_left_hosts',1,'p_version_topology_rule_left_hosts','communication.py',330),
  ('version_topology_rule_left_hosts -> version_topology_host_with_indicies','version_topology_rule_left_hosts',1,'p_version_topology_rule_left_hosts','communication.py',331),
  ('version_topology_rule_right_hosts -> IDENTIFIER','version_topology_rule_right_hosts',1,'p_version_topology_rule_right_hosts','communication.py',340),
  ('version_topology_rule_right_hosts -> STAR','version_topology_rule_right_hosts',1,'p_version_topology_rule_right_hosts','communication.py',341),
  ('version_topology_rule_right_hosts -> version_topology_host_with_indicies','version_topology_rule_right_hosts',1,'p_version_topology_rule_right_hosts','communication.py',342),
  ('version_topology_rule_right_hosts -> version_topology_host_with_i_index','version_topology_rule_right_hosts',1,'p_version_topology_rule_right_hosts','communication.py',343),
  ('version_topology_host_with_indicies -> IDENTIFIER SQLPARAN INTEGER SQRPARAN','version_topology_host_with_indicies',4,'p_version_topology_host_with_indicies','communication.py',355),
  ('version_topology_host_with_indicies -> IDENTIFIER SQLPARAN INTEGER COLON SQRPARAN','version_topology_host_with_indicies',5,'p_version_topology_host_with_indicies','communication.py',356),
  ('version_topology_host_with_indicies -> IDENTIFIER SQLPARAN COLON INTEGER SQRPARAN','version_topology_host_with_indicies',5,'p_version_topology_host_with_indicies','communication.py',357),
  ('version_topology_host_with_indicies -> IDENTIFIER SQLPARAN INTEGER COLON INTEGER SQRPARAN','version_topology_host_with_indicies',6,'p_version_topology_host_with_indicies','communication.py',358),
  ('version_topology_host_with_i_index -> IDENTIFIER SQLPARAN I_INDEX SQRPARAN','version_topology_host_with_i_index',4,'p_version_topology_host_with_i_index','communication.py',374),
  ('version_topology_host_with_i_index -> IDENTIFIER SQLPARAN I_INDEX COMM_PLUS INTEGER SQRPARAN','version_topology_host_with_i_index',6,'p_version_topology_host_with_i_index','communication.py',375),
  ('version_topology_host_with_i_index -> IDENTIFIER SQLPARAN I_INDEX COMM_MINUS INTEGER SQRPARAN','version_topology_host_with_i_index',6,'p_version_topology_host_with_i_index','communication.py',376),
  ('version_topology_rule_parameters -> version_topology_rule_parameter','version_topology_rule_parameters',1,'p_version_topology_rule_parameters','communication.py',391),
  ('version_topology_rule_parameters -> version_topology_rule_parameters COMMA version_topology_rule_parameter','version_topology_rule_parameters',3,'p_version_topology_rule_parameters','communication.py',392),
  ('version_topology_rule_parameter -> Q_PARAMETER EQUAL number','version_topology_rule_parameter',3,'p_version_topology_rule_quality_parameter','communication.py',400),
  ('configuration -> specification','configuration',1,'p_rule_configuration','main.py',402),
  ('configuration -> configuration specification','configuration',2,'p_rule_configuration','main.py',403),
  ('version_topology_arrow -> ARROWRIGHT','version_topology_arrow',1,'p_version_topology_arrow','communication.py',406),
  ('version_topology_arrow -> ARROWLEFT','version_topology_arrow',1,'p_version_topology_arrow','communication.py',407),
  ('version_topology_arrow -> ARROWBOTH','version_topology_arrow',1,'p_version_topology_arrow','communication.py',408),
  ('empty -> <empty>','empty',0,'p_rule_empty','main.py',409),
  ('number -> FLOAT','number',1,'p_rule_number','main.py',415),
  ('number -> INTEGER','number',1,'p_rule_number','main.py',416),
  ('identifiers_list -> IDENTIFIER','identifiers_list',1,'p_rule_identifiers_list','main.py',422),
  ('identifiers_list -> identifiers_list COMMA IDENTIFIER','identifiers_list',3,'p_rule_identifiers_list','main.py',423),
  ('qualified_identifiers_list -> QUALIFIED_IDENTIFIER','qualified_identifiers_list',1,'p_rule_qualified_identifiers_list','main.py',433),
  ('qualified_identifiers_list -> qualified_identifiers_list COMMA QUALIFIED_IDENTIFIER','qualified_identifiers_list',3,'p_rule_qualified_identifiers_list','main.py',434),
]
